@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 # TODO: probably should say which arguments are candidates for glob
@@ -35,6 +35,7 @@ from bzrlib.lazy_import import lazy_import
 lazy_import(globals(), """
 import codecs
 import errno
+import threading
 from warnings import warn
 
 import bzrlib
@@ -50,7 +51,7 @@ from bzrlib import (
 
 from bzrlib import registry
 # Compatibility
-from bzrlib.hooks import Hooks
+from bzrlib.hooks import HookPoint, Hooks
 from bzrlib.option import Option
 
 
@@ -331,7 +332,7 @@ class Command(object):
         return s
 
     def get_help_text(self, additional_see_also=None, plain=True,
-                      see_also_as_links=False):
+                      see_also_as_links=False, verbose=True):
         """Return a text string with help for this command.
 
         :param additional_see_also: Additional help topics to be
@@ -340,6 +341,10 @@ class Command(object):
             returned instead of plain text.
         :param see_also_as_links: if True, convert items in 'See also'
             list to internal links (used by bzr_man rstx generator)
+        :param verbose: if True, display the full help, otherwise
+            leave out the descriptive sections and just display
+            usage help (e.g. Purpose, Usage, Options) with a
+            message explaining how to obtain full help.
         """
         doc = self.help()
         if doc is None:
@@ -374,19 +379,24 @@ class Command(object):
             result += options
         result += '\n'
 
-        # Add the description, indenting it 2 spaces
-        # to match the indentation of the options
-        if sections.has_key(None):
-            text = sections.pop(None)
-            text = '\n  '.join(text.splitlines())
-            result += ':%s:\n  %s\n\n' % ('Description',text)
+        if verbose:
+            # Add the description, indenting it 2 spaces
+            # to match the indentation of the options
+            if sections.has_key(None):
+                text = sections.pop(None)
+                text = '\n  '.join(text.splitlines())
+                result += ':%s:\n  %s\n\n' % ('Description',text)
 
-        # Add the custom sections (e.g. Examples). Note that there's no need
-        # to indent these as they must be indented already in the source.
-        if sections:
-            for label in order:
-                if sections.has_key(label):
-                    result += ':%s:\n%s\n\n' % (label,sections[label])
+            # Add the custom sections (e.g. Examples). Note that there's no need
+            # to indent these as they must be indented already in the source.
+            if sections:
+                for label in order:
+                    if sections.has_key(label):
+                        result += ':%s:\n%s\n' % (label,sections[label])
+                result += '\n'
+        else:
+            result += ("See bzr help %s for more details and examples.\n\n"
+                % self.name())
 
         # Add the aliases, source (plug-in) and see also links, if any
         if self.aliases:
@@ -523,6 +533,9 @@ class Command(object):
         if 'help' in opts:  # e.g. bzr add --help
             sys.stdout.write(self.get_help_text())
             return 0
+        if 'usage' in opts:  # e.g. bzr add --usage
+            sys.stdout.write(self.get_help_text(verbose=False))
+            return 0
         trace.set_verbosity_level(option._verbosity_level)
         if 'verbose' in self.supported_std_options:
             opts['verbose'] = trace.is_verbose()
@@ -591,11 +604,10 @@ class CommandHooks(Hooks):
         notified.
         """
         Hooks.__init__(self)
-        # Introduced in 1.13:
-        # invoked after creating a command object to allow modifications such
-        # as adding or removing options, docs etc. Invoked with the command
-        # object.
-        self['extend_command'] = []
+        self.create_hook(HookPoint('extend_command',
+            "Called after creating a command object to allow modifications "
+            "such as adding or removing options, docs etc. Called with the "
+            "new bzrlib.commands.Command object.", (1, 13), None))
 
 Command.hooks = CommandHooks()
 
@@ -671,6 +683,7 @@ def apply_coveraged(dirname, the_callable, *args, **kwargs):
 
     tracer = trace.Trace(count=1, trace=0)
     sys.settrace(tracer.globaltrace)
+    threading.settrace(tracer.globaltrace)
 
     try:
         return exception_to_return_code(the_callable, *args, **kwargs)
@@ -939,7 +952,16 @@ def display_command(func):
     return ignore_pipe
 
 
-def main(argv):
+def main(argv=None):
+    """Main entry point of command-line interface.
+
+    :param argv: list of unicode command-line arguments similar to sys.argv.
+        argv[0] is script name usually, it will be ignored.
+        Don't pass here sys.argv because this list contains plain strings
+        and not unicode; pass None instead.
+
+    :return: exit code of bzr command.
+    """
     import bzrlib.ui
     bzrlib.ui.ui_factory = bzrlib.ui.make_ui_for_terminal(
         sys.stdin, sys.stdout, sys.stderr)
@@ -948,12 +970,20 @@ def main(argv):
     if bzrlib.version_info[3] == 'final':
         from bzrlib import symbol_versioning
         symbol_versioning.suppress_deprecation_warnings(override=False)
-    try:
-        user_encoding = osutils.get_user_encoding()
-        argv = [a.decode(user_encoding) for a in argv[1:]]
-    except UnicodeDecodeError:
-        raise errors.BzrError(("Parameter '%r' is unsupported by the current "
-                                                            "encoding." % a))
+    if argv is None:
+        argv = osutils.get_unicode_argv()
+    else:
+        new_argv = []
+        try:
+            # ensure all arguments are unicode strings
+            for a in argv[1:]:
+                if isinstance(a, unicode):
+                    new_argv.append(a)
+                else:
+                    new_argv.append(a.decode('ascii'))
+        except UnicodeDecodeError:
+            raise errors.BzrError("argv should be list of unicode strings.")
+        argv = new_argv
     ret = run_bzr_catch_errors(argv)
     trace.mutter("return code %d", ret)
     return ret
