@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Configuration that affects the behaviour of Bazaar.
 
@@ -709,7 +709,6 @@ class BranchConfig(Config):
                         trace.warning('Value "%s" is masked by "%s" from'
                                       ' branch.conf', value, mask_value)
 
-
     def _gpg_signing_command(self):
         """See Config.gpg_signing_command."""
         return self._get_safe_value('_gpg_signing_command')
@@ -917,10 +916,7 @@ class TreeConfig(IniBasedConfig):
     # XXX: Really needs a better name, as this is not part of the tree! -- mbp 20080507
 
     def __init__(self, branch):
-        # XXX: Really this should be asking the branch for its configuration
-        # data, rather than relying on a Transport, so that it can work
-        # more cleanly with a RemoteBranch that has no transport.
-        self._config = TransportConfig(branch._transport, 'branch.conf')
+        self._config = branch._get_config()
         self.branch = branch
 
     def _get_parser(self, file=None):
@@ -993,7 +989,8 @@ class AuthenticationConfig(object):
         section[option_name] = value
         self._save()
 
-    def get_credentials(self, scheme, host, port=None, user=None, path=None):
+    def get_credentials(self, scheme, host, port=None, user=None, path=None, 
+                        realm=None):
         """Returns the matching credentials from authentication.conf file.
 
         :param scheme: protocol
@@ -1005,12 +1002,19 @@ class AuthenticationConfig(object):
         :param user: login (optional)
 
         :param path: the absolute path on the server (optional)
+        
+        :param realm: the http authentication realm (optional)
 
         :return: A dict containing the matching credentials or None.
            This includes:
            - name: the section name of the credentials in the
              authentication.conf file,
-           - user: can't de different from the provided user if any,
+           - user: can't be different from the provided user if any,
+           - scheme: the server protocol,
+           - host: the server address,
+           - port: the server port (can be None),
+           - path: the absolute server path (can be None),
+           - realm: the http specific authentication realm (can be None),
            - password: the decoded password, could be None if the credential
              defines only the user
            - verify_certificates: https specific, True if the server
@@ -1057,20 +1061,35 @@ class AuthenticationConfig(object):
             if a_user is None:
                 # Can't find a user
                 continue
+            # Prepare a credentials dictionary with additional keys
+            # for the credential providers
             credentials = dict(name=auth_def_name,
                                user=a_user,
+                               scheme=a_scheme,
+                               host=host,
+                               port=port,
+                               path=path,
+                               realm=realm,
                                password=auth_def.get('password', None),
                                verify_certificates=a_verify_certificates)
+            # Decode the password in the credentials (or get one)
             self.decode_password(credentials,
                                  auth_def.get('password_encoding', None))
             if 'auth' in debug.debug_flags:
                 trace.mutter("Using authentication section: %r", auth_def_name)
             break
 
+        if credentials is None:
+            # No credentials were found in authentication.conf, try the fallback
+            # credentials stores.
+            credentials = credential_store_registry.get_fallback_credentials(
+                scheme, host, port, user, path, realm)
+
         return credentials
 
     def set_credentials(self, name, host, user, scheme=None, password=None,
-                        port=None, path=None, verify_certificates=None):
+                        port=None, path=None, verify_certificates=None,
+                        realm=None):
         """Set authentication credentials for a host.
 
         Any existing credentials with matching scheme, host, port and path
@@ -1087,6 +1106,7 @@ class AuthenticationConfig(object):
             apply to.
         :param verify_certificates: On https, verify server certificates if
             True.
+        :param realm: The http authentication realm (optional).
         """
         values = {'host': host, 'user': user}
         if password is not None:
@@ -1099,10 +1119,12 @@ class AuthenticationConfig(object):
             values['path'] = path
         if verify_certificates is not None:
             values['verify_certificates'] = str(verify_certificates)
+        if realm is not None:
+            values['realm'] = realm
         config = self._get_config()
         for_deletion = []
         for section, existing_values in config.items():
-            for key in ('scheme', 'host', 'port', 'path'):
+            for key in ('scheme', 'host', 'port', 'path', 'realm'):
                 if existing_values.get(key) != values.get(key):
                     break
             else:
@@ -1110,8 +1132,8 @@ class AuthenticationConfig(object):
         config.update({name: values})
         self._save()
 
-    def get_user(self, scheme, host, port=None,
-                 realm=None, path=None, prompt=None):
+    def get_user(self, scheme, host, port=None, realm=None, path=None,
+                 prompt=None, ask=False, default=None):
         """Get a user from authentication file.
 
         :param scheme: protocol
@@ -1124,14 +1146,32 @@ class AuthenticationConfig(object):
 
         :param path: the absolute path on the server (optional)
 
+        :param ask: Ask the user if there is no explicitly configured username 
+                    (optional)
+
+        :param default: The username returned if none is defined (optional).
+
         :return: The found user.
         """
         credentials = self.get_credentials(scheme, host, port, user=None,
-                                           path=path)
+                                           path=path, realm=realm)
         if credentials is not None:
             user = credentials['user']
         else:
             user = None
+        if user is None:
+            if ask:
+                if prompt is None:
+                    # Create a default prompt suitable for most cases
+                    prompt = scheme.upper() + ' %(host)s username'
+                # Special handling for optional fields in the prompt
+                if port is not None:
+                    prompt_host = '%s:%d' % (host, port)
+                else:
+                    prompt_host = host
+                user = ui.ui_factory.get_username(prompt, host=prompt_host)
+            else:
+                user = default
         return user
 
     def get_password(self, scheme, host, user, port=None,
@@ -1152,7 +1192,8 @@ class AuthenticationConfig(object):
 
         :return: The found password or the one entered by the user.
         """
-        credentials = self.get_credentials(scheme, host, port, user, path)
+        credentials = self.get_credentials(scheme, host, port, user, path,
+                                           realm)
         if credentials is not None:
             password = credentials['password']
             if password is not None and scheme is 'ssh':
@@ -1191,10 +1232,13 @@ class CredentialStoreRegistry(registry.Registry):
     A credential store provides access to credentials via the password_encoding
     field in authentication.conf sections.
 
-    Except for stores provided by bzr itself,most stores are expected to be
+    Except for stores provided by bzr itself, most stores are expected to be
     provided by plugins that will therefore use
     register_lazy(password_encoding, module_name, member_name, help=help,
-    info=info) to install themselves.
+    fallback=fallback) to install themselves.
+
+    A fallback credential store is one that is queried if no credentials can be
+    found via authentication.conf.
     """
 
     def get_credential_store(self, encoding=None):
@@ -1203,6 +1247,68 @@ class CredentialStoreRegistry(registry.Registry):
             cs = cs()
         return cs
 
+    def is_fallback(self, name):
+        """Check if the named credentials store should be used as fallback."""
+        return self.get_info(name)
+
+    def get_fallback_credentials(self, scheme, host, port=None, user=None,
+                                 path=None, realm=None):
+        """Request credentials from all fallback credentials stores.
+
+        The first credentials store that can provide credentials wins.
+        """
+        credentials = None
+        for name in self.keys():
+            if not self.is_fallback(name):
+                continue
+            cs = self.get_credential_store(name)
+            credentials = cs.get_credentials(scheme, host, port, user,
+                                             path, realm)
+            if credentials is not None:
+                # We found some credentials
+                break
+        return credentials
+
+    def register(self, key, obj, help=None, override_existing=False,
+                 fallback=False):
+        """Register a new object to a name.
+
+        :param key: This is the key to use to request the object later.
+        :param obj: The object to register.
+        :param help: Help text for this entry. This may be a string or
+                a callable. If it is a callable, it should take two
+                parameters (registry, key): this registry and the key that
+                the help was registered under.
+        :param override_existing: Raise KeyErorr if False and something has
+                already been registered for that key. If True, ignore if there
+                is an existing key (always register the new value).
+        :param fallback: Whether this credential store should be 
+                used as fallback.
+        """
+        return super(CredentialStoreRegistry,
+                     self).register(key, obj, help, info=fallback,
+                                    override_existing=override_existing)
+
+    def register_lazy(self, key, module_name, member_name,
+                      help=None, override_existing=False,
+                      fallback=False):
+        """Register a new credential store to be loaded on request.
+
+        :param module_name: The python path to the module. Such as 'os.path'.
+        :param member_name: The member of the module to return.  If empty or
+                None, get() will return the module itself.
+        :param help: Help text for this entry. This may be a string or
+                a callable.
+        :param override_existing: If True, replace the existing object
+                with the new one. If False, if there is already something
+                registered with the same key, raise a KeyError
+        :param fallback: Whether this credential store should be 
+                used as fallback.
+        """
+        return super(CredentialStoreRegistry, self).register_lazy(
+            key, module_name, member_name, help,
+            info=fallback, override_existing=override_existing)
+
 
 credential_store_registry = CredentialStoreRegistry()
 
@@ -1211,9 +1317,18 @@ class CredentialStore(object):
     """An abstract class to implement storage for credentials"""
 
     def decode_password(self, credentials):
-        """Returns a password for the provided credentials in clear text."""
+        """Returns a clear text password for the provided credentials."""
         raise NotImplementedError(self.decode_password)
 
+    def get_credentials(self, scheme, host, port=None, user=None, path=None,
+                        realm=None):
+        """Return the matching credentials from this credential store.
+
+        This method is only called on fallback credential stores.
+        """
+        raise NotImplementedError(self.get_credentials)
+
+
 
 class PlainTextCredentialStore(CredentialStore):
     """Plain text credential store for the authentication.conf file."""
@@ -1230,8 +1345,9 @@ credential_store_registry.default_key = 'plain'
 
 class BzrDirConfig(object):
 
-    def __init__(self, transport):
-        self._config = TransportConfig(transport, 'control.conf')
+    def __init__(self, bzrdir):
+        self._bzrdir = bzrdir
+        self._config = bzrdir._get_config()
 
     def set_default_stack_on(self, value):
         """Set the default stacking location.
@@ -1241,6 +1357,8 @@ class BzrDirConfig(object):
         This policy affects all branches contained by this bzrdir, except for
         those under repositories.
         """
+        if self._config is None:
+            raise errors.BzrError("Cannot set configuration in %s" % self._bzrdir)
         if value is None:
             self._config.set_option('', 'default_stack_on')
         else:
@@ -1254,6 +1372,8 @@ class BzrDirConfig(object):
         This policy affects all branches contained by this bzrdir, except for
         those under repositories.
         """
+        if self._config is None:
+            return None
         value = self._config.get_option('default_stack_on')
         if value == '':
             value = None
@@ -1304,12 +1424,14 @@ class TransportConfig(object):
             configobj.setdefault(section, {})[name] = value
         self._set_configobj(configobj)
 
-    def _get_configobj(self):
+    def _get_config_file(self):
         try:
-            return ConfigObj(self._transport.get(self._filename),
-                             encoding='utf-8')
+            return self._transport.get(self._filename)
         except errors.NoSuchFile:
-            return ConfigObj(encoding='utf-8')
+            return StringIO()
+
+    def _get_configobj(self):
+        return ConfigObj(self._get_config_file(), encoding='utf-8')
 
     def _set_configobj(self, configobj):
         out_file = StringIO()
