@@ -1,4 +1,4 @@
-# Copyright (C) 2005, 2006, 2007, 2008 Canonical Ltd
+# Copyright (C) 2005, 2006, 2007, 2008, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Locking using OS file locks or file existence.
@@ -42,21 +42,22 @@ from bzrlib import (
     osutils,
     trace,
     )
-from bzrlib.hooks import Hooks
+from bzrlib.hooks import HookPoint, Hooks
 
 
 class LockHooks(Hooks):
 
     def __init__(self):
         Hooks.__init__(self)
-
-        # added in 1.8; called with a LockResult when a physical lock is
-        # acquired
-        self['lock_acquired'] = []
-
-        # added in 1.8; called with a LockResult when a physical lock is
-        # acquired
-        self['lock_released'] = []
+        self.create_hook(HookPoint('lock_acquired',
+            "Called with a bzrlib.lock.LockResult when a physical lock is "
+            "acquired.", (1, 8), None))
+        self.create_hook(HookPoint('lock_released',
+            "Called with a bzrlib.lock.LockResult when a physical lock is "
+            "released.", (1, 8), None))
+        self.create_hook(HookPoint('lock_broken',
+            "Called with a bzrlib.lock.LockResult when a physical lock is "
+            "broken.", (1, 15), None))
 
 
 class Lock(object):
@@ -79,6 +80,10 @@ class LockResult(object):
     def __eq__(self, other):
         return self.lock_url == other.lock_url and self.details == other.details
 
+    def __repr__(self):
+        return '%s(%s%s)' % (self.__class__.__name__,
+                             self.lock_url, self.details)
+
 
 try:
     import fcntl
@@ -185,7 +190,7 @@ if have_fcntl:
                     self.unlock()
                 # we should be more precise about whats a locking
                 # error and whats a random-other error
-                raise errors.LockContention(e)
+                raise errors.LockContention(self.filename, e)
 
         def unlock(self):
             _fcntl_WriteLock._open_locks.remove(self.filename)
@@ -209,7 +214,7 @@ if have_fcntl:
             except IOError, e:
                 # we should be more precise about whats a locking
                 # error and whats a random-other error
-                raise errors.LockContention(e)
+                raise errors.LockContention(self.filename, e)
 
         def unlock(self):
             count = _fcntl_ReadLock._open_locks[self.filename]
@@ -277,7 +282,7 @@ if have_fcntl:
                 fcntl.lockf(new_f, fcntl.LOCK_EX | fcntl.LOCK_NB)
             except IOError, e:
                 # TODO: Raise a more specific error based on the type of error
-                raise errors.LockContention(e)
+                raise errors.LockContention(self.filename, e)
             _fcntl_WriteLock._open_locks.add(self.filename)
 
             self.f = new_f
@@ -322,7 +327,7 @@ if have_pywin32 and sys.platform == 'win32':
                 raise
             except Exception, e:
                 self._clear_f()
-                raise errors.LockContention(e)
+                raise errors.LockContention(filename, e)
 
         def unlock(self):
             overlapped = pywintypes.OVERLAPPED()
@@ -330,7 +335,7 @@ if have_pywin32 and sys.platform == 'win32':
                 win32file.UnlockFileEx(self.hfile, 0, 0x7fff0000, overlapped)
                 self._clear_f()
             except Exception, e:
-                raise errors.LockContention(e)
+                raise errors.LockContention(self.filename, e)
 
 
     class _w32c_ReadLock(_w32c_FileLock):
@@ -386,7 +391,6 @@ if have_ctypes_win32:
     LOCK_NB = LOCKFILE_FAIL_IMMEDIATELY
     _LockFileEx = ctypes.windll.kernel32.LockFileEx
     _UnlockFileEx = ctypes.windll.kernel32.UnlockFileEx
-    _GetLastError = ctypes.windll.kernel32.GetLastError
 
     ### Define the OVERLAPPED structure.
     #   http://msdn2.microsoft.com/en-us/library/ms684342.aspx
@@ -435,12 +439,12 @@ if have_ctypes_win32:
                                  ctypes.byref(overlapped), # lpOverlapped
                                 )
             if result == 0:
+                last_err = ctypes.GetLastError()
                 self._clear_f()
-                last_err = _GetLastError()
                 if last_err in (ERROR_LOCK_VIOLATION,):
                     raise errors.LockContention(filename)
-                raise errors.LockContention('Unknown locking error: %s'
-                                            % (last_err,))
+                raise errors.LockContention(filename,
+                    'Unknown locking error: %s' % (last_err,))
 
         def unlock(self):
             overlapped = OVERLAPPED()
@@ -450,12 +454,12 @@ if have_ctypes_win32:
                                    0x00000000, # DWORD nNumberOfBytesToLockHigh
                                    ctypes.byref(overlapped), # lpOverlapped
                                   )
-            self._clear_f()
             if result == 0:
+                last_err = ctypes.GetLastError()
                 self._clear_f()
-                last_err = _GetLastError()
-                raise errors.LockContention('Unknown unlocking error: %s'
-                                            % (last_err,))
+                raise errors.LockContention(self.filename,
+                    'Unknown unlocking error: %s' % (last_err,))
+            self._clear_f()
 
 
     class _ctypes_ReadLock(_ctypes_FileLock):
