@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Progress indicators.
@@ -36,11 +36,13 @@ from bzrlib import (
     )
 from bzrlib.trace import mutter
 from bzrlib.symbol_versioning import (
+    deprecated_function,
     deprecated_in,
     deprecated_method,
     )
 
 
+# XXX: deprecated; can be removed when the ProgressBar factory is removed
 def _supports_progress(f):
     """Detect if we can use pretty progress bars on the output stream f.
 
@@ -70,6 +72,11 @@ class ProgressTask(object):
     """
 
     def __init__(self, parent_task=None, ui_factory=None):
+        """Construct a new progress task.
+
+        Normally you should not call this directly but rather through
+        `ui_factory.nested_progress_bar`.
+        """
         self._parent_task = parent_task
         self._last_update = 0
         self.total_cnt = None
@@ -112,7 +119,9 @@ class ProgressTask(object):
         if self.current_cnt is not None and self.total_cnt:
             own_fraction = (float(self.current_cnt) + child_fraction) / self.total_cnt
         else:
-            own_fraction = None
+            # if this task has no estimation, it just passes on directly
+            # whatever the child has measured...
+            own_fraction = child_fraction
         if self._parent_task is None:
             return own_fraction
         else:
@@ -133,6 +142,7 @@ class ProgressTask(object):
         self.ui_factory.clear_term()
 
 
+@deprecated_function(deprecated_in((1, 16, 0)))
 def ProgressBar(to_file=None, **kwargs):
     """Abstract factory"""
     if to_file is None:
@@ -156,75 +166,6 @@ def ProgressBar(to_file=None, **kwargs):
         return _progress_bar_types[requested_bar_type](to_file=to_file, **kwargs)
 
 
-class ProgressBarStack(object):
-    """A stack of progress bars.
-
-    This class is deprecated: instead, ask the ui factory for a new progress
-    task and finish it when it's done.
-    """
-
-    @deprecated_method(deprecated_in((1, 12, 0)))
-    def __init__(self,
-                 to_file=None,
-                 show_pct=False,
-                 show_spinner=True,
-                 show_eta=False,
-                 show_bar=True,
-                 show_count=True,
-                 to_messages_file=None,
-                 klass=None):
-        """Setup the stack with the parameters the progress bars should have."""
-        if to_file is None:
-            to_file = sys.stderr
-        if to_messages_file is None:
-            to_messages_file = sys.stdout
-        self._to_file = to_file
-        self._show_pct = show_pct
-        self._show_spinner = show_spinner
-        self._show_eta = show_eta
-        self._show_bar = show_bar
-        self._show_count = show_count
-        self._to_messages_file = to_messages_file
-        self._stack = []
-        self._klass = klass or ProgressBar
-
-    def top(self):
-        if len(self._stack) != 0:
-            return self._stack[-1]
-        else:
-            return None
-
-    def bottom(self):
-        if len(self._stack) != 0:
-            return self._stack[0]
-        else:
-            return None
-
-    def get_nested(self):
-        """Return a nested progress bar."""
-        if len(self._stack) == 0:
-            func = self._klass
-        else:
-            func = self.top().child_progress
-        new_bar = func(to_file=self._to_file,
-                       show_pct=self._show_pct,
-                       show_spinner=self._show_spinner,
-                       show_eta=self._show_eta,
-                       show_bar=self._show_bar,
-                       show_count=self._show_count,
-                       to_messages_file=self._to_messages_file,
-                       _stack=self)
-        self._stack.append(new_bar)
-        return new_bar
-
-    def return_pb(self, bar):
-        """Return bar after its been used."""
-        if bar is not self._stack[-1]:
-            warnings.warn("%r is not currently active" % (bar,))
-        else:
-            self._stack.pop()
-
-
 class _BaseProgressBar(object):
 
     def __init__(self,
@@ -271,6 +212,7 @@ class _BaseProgressBar(object):
         self.to_messages_file.write(fmt_string % args)
         self.to_messages_file.write('\n')
 
+    @deprecated_function(deprecated_in((1, 16, 0)))
     def child_progress(self, **kwargs):
         return ChildProgress(**kwargs)
 
@@ -302,6 +244,7 @@ class DummyProgress(_BaseProgressBar):
 
 class DotsProgressBar(_BaseProgressBar):
 
+    @deprecated_function(deprecated_in((1, 16, 0)))
     def __init__(self, **kwargs):
         _BaseProgressBar.__init__(self, **kwargs)
         self.last_msg = None
@@ -328,8 +271,6 @@ class DotsProgressBar(_BaseProgressBar):
         self.tick()
 
 
-
-
 class TTYProgressBar(_BaseProgressBar):
     """Progress bar display object.
 
@@ -352,7 +293,7 @@ class TTYProgressBar(_BaseProgressBar):
     """
     SPIN_CHARS = r'/-\|'
 
-
+    @deprecated_function(deprecated_in((1, 16, 0)))
     def __init__(self, **kwargs):
         from bzrlib.osutils import terminal_width
         _BaseProgressBar.__init__(self, **kwargs)
@@ -512,11 +453,10 @@ class TTYProgressBar(_BaseProgressBar):
         #self.to_file.flush()
 
 
-
-
 class ChildProgress(_BaseProgressBar):
     """A progress indicator that pushes its data to the parent"""
 
+    @deprecated_function(deprecated_in((1, 16, 0)))
     def __init__(self, _stack, **kwargs):
         _BaseProgressBar.__init__(self, _stack=_stack, **kwargs)
         self.parent = _stack.top()
@@ -558,23 +498,6 @@ class ChildProgress(_BaseProgressBar):
         self.parent.note(*args, **kwargs)
 
 
-class InstrumentedProgress(TTYProgressBar):
-    """TTYProgress variant that tracks outcomes"""
-
-    def __init__(self, *args, **kwargs):
-        self.always_throttled = True
-        self.never_throttle = False
-        TTYProgressBar.__init__(self, *args, **kwargs)
-
-    def throttle(self, old_message):
-        if self.never_throttle:
-            result =  False
-        else:
-            result = TTYProgressBar.throttle(self, old_message)
-        if result is False:
-            self.always_throttled = False
-
-
 def str_tdelta(delt):
     if delt is None:
         return "-:--:--"
