@@ -12,17 +12,21 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Blackbox tests for debugger breakin"""
 
+import errno
 import os
 import signal
 import subprocess
 import sys
 import time
 
-from bzrlib import tests
+from bzrlib import (
+    errors,
+    tests,
+    )
 
 
 class TestBreakin(tests.TestCase):
@@ -44,6 +48,34 @@ class TestBreakin(tests.TestCase):
             raise tests.TestNotApplicable(
                 '%s raises a popup on OSX' % self.id())
 
+    def _wait_for_process(self, pid, sig=None):
+        # We don't know quite how long waiting for the process 'pid' will take,
+        # but if it's more than 10s then it's probably not going to work.
+        for i in range(100):
+            time.sleep(0.1)
+            if sig is not None:
+                os.kill(pid, sig)
+            # Use WNOHANG to ensure we don't get blocked, doing so, we may
+            # leave the process continue after *we* die...
+            try:
+                # note: waitpid is different on win32, but this test only runs
+                # on unix
+                pid_killed, returncode = os.waitpid(pid, os.WNOHANG)
+                if (pid_killed, returncode) != (0, 0):
+                    if sig is not None:
+                        # high bit in low byte says if core was dumped; we
+                        # don't care
+                        status, sig = (returncode >> 8, returncode & 0x7f)
+                        return True, sig
+            except OSError, e:
+                if e.errno in (errno.ECHILD, errno.ESRCH):
+                    # The process doesn't exist anymore
+                    return True, None
+                else:
+                    raise
+
+        return False, None
+
     # port 0 means to allocate any port
     _test_process_args = ['serve', '--port', 'localhost:0']
 
@@ -57,12 +89,26 @@ class TestBreakin(tests.TestCase):
         proc.stderr.readline()
         # first sigquit pops into debugger
         os.kill(proc.pid, signal.SIGQUIT)
-        proc.stdin.write("q\n")
-        time.sleep(.5)
+        # Wait for the debugger to acknowledge the signal reception
         err = proc.stderr.readline()
         self.assertContainsRe(err, r'entering debugger')
+        # Now that the debugger is entered, we can ask him to quit
+        proc.stdin.write("q\n")
+        # We wait a bit to let the child process handles our query and avoid
+        # triggering deadlocks leading to hangs on multi-core hosts...
+        dead, sig = self._wait_for_process(proc.pid)
+        if not dead:
+            # The process didn't finish, let's kill it before reporting failure
+            dead, sig = self._wait_for_process(proc.pid, signal.SIGKILL)
+            if dead:
+                raise tests.KnownFailure(
+                    "subprocess wasn't terminated, it had to be killed")
+            else:
+                self.fail("subprocess %d wasn't terminated by repeated SIGKILL",
+                          proc.pid)
 
     def test_breakin_harder(self):
+        """SIGQUITting twice ends the process."""
         self._dont_SIGQUIT_on_darwin()
         proc = self.start_bzr_subprocess(self._test_process_args,
                 env_changes=dict(BZR_SIGQUIT_PDB=None))
@@ -70,23 +116,14 @@ class TestBreakin(tests.TestCase):
         proc.stderr.readline()
         # break into the debugger
         os.kill(proc.pid, signal.SIGQUIT)
-        # now send a second sigquit, which should cause it to exit.  That
-        # won't happen until the original signal has been noticed by the
-        # child and it's run its signal handler.  We don't know quite how long
-        # this will take, but if it's more than 10s then it's probably not
-        # going to work.
-        for i in range(100):
-            time.sleep(0.1)
-            os.kill(proc.pid, signal.SIGQUIT)
-            # note: waitpid is different on win32, but this test only runs on
-            # unix
-            r = os.waitpid(proc.pid, os.WNOHANG)
-            if r != (0, 0):
-                # high bit says if core was dumped; we don't care
-                self.assertEquals(r[1] & 0x7f, signal.SIGQUIT)
-                break
-        else:
-            self.fail("subprocess wasn't terminated by repeated SIGQUIT")
+        # Wait for the debugger to acknowledge the signal reception (since we
+        # want to send a second signal, we ensure it doesn't get lost by
+        # validating the first get received and produce its effect).
+        err = proc.stderr.readline()
+        self.assertContainsRe(err, r'entering debugger')
+        dead, sig = self._wait_for_process(proc.pid, signal.SIGQUIT)
+        self.assertTrue((dead and sig == signal.SIGQUIT),
+                        msg="subprocess wasn't terminated by repeated SIGQUIT")
 
     def test_breakin_disabled(self):
         self._dont_SIGQUIT_on_darwin()
