@@ -1,4 +1,4 @@
-# Copyright (C) 2006, 2007 Canonical Ltd
+# Copyright (C) 2006, 2007, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Test "bzr init"""
@@ -49,6 +49,13 @@ class TestInit(ExternalBase):
             out)
         self.assertEqual('', err)
 
+    def test_init_format_2a(self):
+        """Smoke test for constructing a format 2a repoistory."""
+        out, err = self.run_bzr('init --format=2a')
+        self.assertEqual("""Created a standalone tree (format: 2a)\n""",
+            out)
+        self.assertEqual('', err)
+
     def test_init_at_repository_root(self):
         # bzr init at the root of a repository should create a branch
         # and working tree even when creation of working trees is disabled.
