@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Black-box tests for bzr pull."""
@@ -239,7 +239,8 @@ class TestPull(ExternalBase):
         out = self.run_bzr('pull ../branch_a', retcode=3)
         self.assertEqual(out,
                 ('','bzr: ERROR: These branches have diverged.'
-                    ' Use the merge command to reconcile them.\n'))
+                    ' Use the missing command to see how.\n'
+                    'Use the merge command to reconcile them.\n'))
         self.assertEqual(branch_b.get_parent(), parent)
         # test implicit --remember after resolving previous failure
         uncommit(branch=branch_b, tree=tree_b)
@@ -358,3 +359,31 @@ class TestPull(ExternalBase):
         self.assertContainsRe(out, r'\n {4}1 .*\n {6}setup\n')
         self.assertNotContainsRe(
             out, r'revno: 1\ncommitter: .*\nbranch nick: source')
+
+    def test_pull_smart_stacked_streaming_acceptance(self):
+        """'bzr pull -r 123' works on stacked, smart branches, even when the
+        revision specified by the revno is only present in the fallback
+        repository.
+
+        See <https://launchpad.net/bugs/380314>
+        """
+        self.setup_smart_server_with_call_log()
+        parent = self.make_branch_and_tree('parent', format='1.9')
+        parent.commit(message='first commit')
+        local = parent.bzrdir.sprout('local').open_workingtree()
+        local.commit(message='local commit')
+        local.branch.create_clone_on_transport(
+            self.get_transport('stacked'), stacked_on=self.get_url('parent'))
+        empty = self.make_branch_and_tree('empty', format='1.9')
+        self.reset_smart_call_log()
+        self.run_bzr(['pull', '-r', '1', self.get_url('stacked')],
+            working_dir='empty')
+        # This figure represent the amount of work to perform this use case. It
+        # is entirely ok to reduce this number if a test fails due to rpc_count
+        # being too low. If rpc_count increases, more network roundtrips have
+        # become necessary for this use case. Please do not adjust this number
+        # upwards without agreement from bzr's network support maintainers.
+        self.assertLength(43, self.hpss_calls)
+        remote = Branch.open('stacked')
+        self.assertEndsWith(remote.get_stacked_on_url(), '/parent')
+
