@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2005 Canonical Ltd
 # -*- coding: utf-8 -*-
 #
 # This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 """Black-box tests for bzr revno.
@@ -37,17 +37,17 @@ class TestRevno(TestCaseInTempDir):
         self.assertEquals(int(bzr('revno')), 0)
 
         open('foo', 'wb').write('foo\n')
-        bzr('add', 'foo')
-        bzr('commit', '-m', 'foo')
+        bzr('add foo')
+        bzr('commit -m foo')
         self.assertEquals(int(bzr('revno')), 1)
 
         os.mkdir('baz')
-        bzr('add', 'baz')
-        bzr('commit', '-m', 'baz')
+        bzr('add baz')
+        bzr('commit -m baz')
         self.assertEquals(int(bzr('revno')), 2)
 
         os.chdir('..')
-        self.assertEquals(int(bzr('revno', 'a')), 2)
-        self.assertEquals(int(bzr('revno', 'a/baz')), 2)
+        self.assertEquals(int(bzr('revno a')), 2)
+        self.assertEquals(int(bzr('revno a/baz')), 2)
 
 
