@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for branch.push behaviour."""
 
@@ -274,18 +274,13 @@ class TestPush(TestCaseWithBranch):
         # Tweak some class variables to stop remote get_parent_map calls asking
         # for or receiving more data than the caller asked for.
         old_flag = SmartServerRepositoryGetParentMap.no_extra_results
-        inter_classes = [repository.InterOtherToRemote,
-            repository.InterPackToRemotePack]
-        old_batch_sizes = []
-        for inter_class in inter_classes:
-            old_batch_sizes.append(
-                inter_class._walk_to_common_revisions_batch_size)
-            inter_class._walk_to_common_revisions_batch_size = 1
+        inter_class = repository.InterRepository
+        old_batch_size = inter_class._walk_to_common_revisions_batch_size
+        inter_class._walk_to_common_revisions_batch_size = 1
         SmartServerRepositoryGetParentMap.no_extra_results = True
         def reset_values():
             SmartServerRepositoryGetParentMap.no_extra_results = old_flag
-            for inter_class, size in zip(inter_classes, old_batch_sizes):
-                inter_class._walk_to_common_revisions_batch_size = size
+            inter_class._walk_to_common_revisions_batch_size = old_batch_size
         self.addCleanup(reset_values)
 
 
@@ -441,3 +436,13 @@ class EmptyPushSmartEffortTests(TestCaseWithBranch):
         self.assertTrue(len(self.hpss_calls) <= 9, self.hpss_calls)
 
 
+class TestLossyPush(TestCaseWithBranch):
+
+    def setUp(self):
+        self.hook_calls = []
+        TestCaseWithBranch.setUp(self)
+
+    def test_lossy_push_raises_same_vcs(self):
+        target = self.make_branch('target')
+        source = self.make_branch('source')
+        self.assertRaises(errors.LossyPushToSameVCS, source.lossy_push, target)
