@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for Branch.get_stacked_on_url and set_stacked_on_url."""
 
@@ -126,14 +126,6 @@ class TestStacking(TestCaseWithBranch):
         self.assertRevisionNotInRepository('mainline', new_branch_revid)
         self.assertRevisionInRepository('newbranch', new_branch_revid)
 
-    # XXX: this helper probably belongs on TestCaseWithTransport
-    def make_smart_server(self, path):
-        smart_server = server.SmartTCPServer_for_testing()
-        smart_server.setUp(self.get_server())
-        remote_transport = get_transport(smart_server.get_url()).clone(path)
-        self.addCleanup(smart_server.tearDown)
-        return remote_transport
-
     def test_sprout_stacked_from_smart_server(self):
         if isinstance(self.branch_format, branch.BzrBranchFormat4):
             raise TestNotApplicable('Branch format 4 is not usable via HPSS.')
@@ -429,6 +421,31 @@ class TestStacking(TestCaseWithBranch):
         # Ensure that opening the branch doesn't raise.
         branch.Branch.open(transport.base)
 
+    def test_revision_history_of_stacked(self):
+        # See <https://launchpad.net/bugs/380314>.
+        stack_on = self.make_branch_and_tree('stack-on')
+        stack_on.commit('first commit', rev_id='rev1')
+        try:
+            stacked_dir = stack_on.bzrdir.sprout(
+                self.get_url('stacked'), stacked=True)
+        except unstackable_format_errors, e:
+            raise TestNotApplicable('Format does not support stacking.')
+        try:
+            stacked = stacked_dir.open_workingtree()
+        except errors.NoWorkingTree:
+            stacked = stacked_dir.open_branch().create_checkout(
+                'stacked-checkout', lightweight=True)
+        stacked.commit('second commit', rev_id='rev2')
+        # Sanity check: stacked's repo should not contain rev1, otherwise this
+        # test isn't testing what it's supposed to.
+        repo = stacked.branch.repository.bzrdir.open_repository()
+        repo.lock_read()
+        self.addCleanup(repo.unlock)
+        self.assertEqual({}, repo.get_parent_map(['rev1']))
+        # revision_history should work, even though the history is spread over
+        # multiple repositories.
+        self.assertLength(2, stacked.branch.revision_history())
+
 
 class TestStackingConnections(
     transport_util.TestCaseWithConnectionHookedTransport):
