@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for InterRepository implementastions."""
 
@@ -27,7 +27,6 @@ from bzrlib.inventory import Inventory
 import bzrlib.repofmt.weaverepo as weaverepo
 import bzrlib.repository as repository
 from bzrlib.revision import NULL_REVISION, Revision
-from bzrlib.symbol_versioning import one_two
 from bzrlib.tests import (
     TestCase,
     TestCaseWithTransport,
@@ -115,19 +114,6 @@ class TestCaseWithComplexRepository(TestCaseWithInterRepository):
         tree_a.branch.repository.commit_write_group()
         tree_a.branch.repository.unlock()
 
-    def test_missing_revision_ids_is_deprecated(self):
-        repo_b = self.make_to_repository('rev1_only')
-        repo_a = self.bzrdir.open_repository()
-        repo_b.fetch(repo_a, 'rev1')
-        # check the test will be valid
-        self.assertFalse(repo_b.has_revision('rev2'))
-        self.assertEqual(['rev2'],
-            self.applyDeprecated(one_two, repo_b.missing_revision_ids, repo_a))
-        inter = repository.InterRepository.get(repo_a, repo_b)
-        self.assertEqual(['rev2'],
-            self.applyDeprecated(one_two, inter.missing_revision_ids, None,
-                True))
-
     def test_search_missing_revision_ids(self):
         # revision ids in repository A but not B are returned, fake ones
         # are stripped. (fake meaning no revision object, but an inventory
@@ -140,7 +126,8 @@ class TestCaseWithComplexRepository(TestCaseWithInterRepository):
         self.assertFalse(repo_b.has_revision('rev2'))
         result = repo_b.search_missing_revision_ids(repo_a)
         self.assertEqual(set(['rev2']), result.get_keys())
-        self.assertEqual((set(['rev2']), set(['rev1']), 1), result.get_recipe())
+        self.assertEqual(('search', set(['rev2']), set(['rev1']), 1),
+            result.get_recipe())
 
     def test_search_missing_revision_ids_absent_requested_raises(self):
         # Asking for missing revisions with a tip that is itself absent in the
@@ -166,7 +153,7 @@ class TestCaseWithComplexRepository(TestCaseWithInterRepository):
         repo_a = self.bzrdir.open_repository()
         result = repo_b.search_missing_revision_ids(repo_a, revision_id='rev1')
         self.assertEqual(set(['rev1']), result.get_keys())
-        self.assertEqual((set(['rev1']), set([NULL_REVISION]), 1),
+        self.assertEqual(('search', set(['rev1']), set([NULL_REVISION]), 1),
             result.get_recipe())
 
     def test_fetch_fetches_signatures_too(self):
