@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for the compiled dirstate helpers."""
 
@@ -23,28 +23,72 @@ import time
 from bzrlib import (
     dirstate,
     errors,
+    osutils,
     tests,
     )
 from bzrlib.tests import (
-        SymlinkFeature,
-        )
-from bzrlib.tests import test_dirstate
+    test_dirstate,
+    test_osutils,
+    )
+
+try:
+    from bzrlib import _dirstate_helpers_c
+    has_dirstate_helpers_c = True
+except ImportError:
+    has_dirstate_helpers_c = False
 
 
 class _CompiledDirstateHelpersFeature(tests.Feature):
     def _probe(self):
-        try:
-            import bzrlib._dirstate_helpers_c
-        except ImportError:
-            return False
-        return True
+        return has_dirstate_helpers_c
 
     def feature_name(self):
         return 'bzrlib._dirstate_helpers_c'
-
 CompiledDirstateHelpersFeature = _CompiledDirstateHelpersFeature()
 
 
+def load_tests(basic_tests, module, loader):
+    # FIXME: we should also parametrize against SHA1Provider !
+    suite = loader.suiteClass()
+    remaining_tests = basic_tests
+
+    dir_reader_scenarios = test_osutils.dir_reader_scenarios()
+
+    ue_scenarios = [('dirstate_Python',
+                     {'update_entry': dirstate.py_update_entry})]
+    if has_dirstate_helpers_c:
+        c_scenario = ('dirstate_C',
+                     {'update_entry': _dirstate_helpers_c.update_entry})
+        ue_scenarios.append(c_scenario)
+    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance(TestUpdateEntry))
+    tests.multiply_tests(process_entry_tests,
+                         tests.multiply_scenarios(dir_reader_scenarios,
+                                                  ue_scenarios),
+                         suite)
+
+    pe_scenarios = [('dirstate_Python',
+                     {'_process_entry': dirstate.ProcessEntryPython})]
+    if has_dirstate_helpers_c:
+        c_scenario = ('dirstate_C',
+                     {'_process_entry': _dirstate_helpers_c.ProcessEntryC})
+        pe_scenarios.append(c_scenario)
+    process_entry_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance(TestProcessEntry))
+    tests.multiply_tests(process_entry_tests,
+                         tests.multiply_scenarios(dir_reader_scenarios,
+                                                  pe_scenarios),
+                         suite)
+
+    dir_reader_tests, remaining_tests = tests.split_suite_by_condition(
+        remaining_tests, tests.condition_isinstance(
+            test_dirstate.TestCaseWithDirState))
+    tests.multiply_tests(dir_reader_tests, dir_reader_scenarios, suite)
+    suite.addTest(remaining_tests)
+
+    return suite
+
+
 class TestBisectPathMixin(object):
     """Test that _bisect_path_*() returns the expected values.
 
@@ -789,18 +833,25 @@ class TestUsingCompiledIfAvailable(tests.TestCase):
 class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
     """Test the DirState.update_entry functions"""
 
+    # Set by load_tests
+    update_entry = None
+
+    def setUp(self):
+        super(TestUpdateEntry, self).setUp()
+        orig = dirstate.update_entry
+        def cleanup():
+            dirstate.update_entry = orig
+        self.addCleanup(cleanup)
+        dirstate.update_entry = self.update_entry
+
     def get_state_with_a(self):
         """Create a DirState tracking a single object named 'a'"""
         state = test_dirstate.InstrumentedDirState.initialize('dirstate')
         self.addCleanup(state.unlock)
         state.add('a', 'a-id', 'file', None, '')
         entry = state._get_entry(0, path_utf8='a')
-        self.set_update_entry()
         return state, entry
 
-    def set_update_entry(self):
-        self.update_entry = dirstate.py_update_entry
-
     def test_observed_sha1_cachable(self):
         state, entry = self.get_state_with_a()
         atime = time.time() - 10
@@ -920,7 +971,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
     def test_update_entry_symlink(self):
         """Update entry should read symlinks."""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         state.save()
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
@@ -1020,6 +1071,16 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
         self.assertEqual(dirstate.DirState.IN_MEMORY_UNMODIFIED,
                          state._dirblock_state)
 
+    def test_update_entry_tree_reference(self):
+        state = test_dirstate.InstrumentedDirState.initialize('dirstate')
+        self.addCleanup(state.unlock)
+        state.add('r', 'r-id', 'tree-reference', None, '')
+        self.build_tree(['r/'])
+        entry = state._get_entry(0, path_utf8='r')
+        self.do_update_entry(state, entry, 'r')
+        entry = state._get_entry(0, path_utf8='r')
+        self.assertEqual('t', entry[1][0][0])
+
     def create_and_test_file(self, state, entry):
         """Create a file at 'a' and verify the state finds it during update.
 
@@ -1052,6 +1113,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
         return packed_stat
 
+    # FIXME: Add unicode version
     def create_and_test_symlink(self, state, entry):
         """Create a symlink at 'a' and verify the state finds it.
 
@@ -1086,7 +1148,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
     def test_update_file_to_symlink(self):
         """File becomes a symlink"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The file sha1 won't be cached unless the file is old
         state.adjust_time(+10)
@@ -1105,7 +1167,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
     def test_update_dir_to_symlink(self):
         """Directory becomes a symlink"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1115,7 +1177,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
     def test_update_symlink_to_file(self):
         """Symlink becomes a file"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink and file info won't be cached unless old
         state.adjust_time(+10)
@@ -1125,7 +1187,7 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
 
     def test_update_symlink_to_dir(self):
         """Symlink becomes a directory"""
-        self.requireFeature(SymlinkFeature)
+        self.requireFeature(tests.SymlinkFeature)
         state, entry = self.get_state_with_a()
         # The symlink target won't be cached if it isn't old
         state.adjust_time(+10)
@@ -1154,20 +1216,106 @@ class TestUpdateEntry(test_dirstate.TestCaseWithDirState):
         self.assertEqual([('f', '', 14, True, dirstate.DirState.NULLSTAT)],
                          entry[1])
 
-        # Make the disk object look old enough to cache (but it won't cache the sha
-        # as it is a new file).
+        # Make the disk object look old enough to cache (but it won't cache the
+        # sha as it is a new file).
         state.adjust_time(+20)
         digest = 'b50e5406bb5e153ebbeb20268fcf37c87e1ecfb6'
         self.update_entry(state, entry, abspath='a', stat_value=stat_value)
         self.assertEqual([('f', '', 14, True, dirstate.DirState.NULLSTAT)],
             entry[1])
 
+    def _prepare_tree(self):
+        # Create a tree
+        text = 'Hello World\n'
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree_contents([('tree/a file', text)])
+        tree.add('a file', 'a-file-id')
+        # Note: dirstate does not sha prior to the first commit
+        # so commit now in order for the test to work
+        tree.commit('first')
+        return tree, text
 
-class TestCompiledUpdateEntry(TestUpdateEntry):
-    """Test the pyrex implementation of _read_dirblocks"""
+    def test_sha1provider_sha1_used(self):
+        tree, text = self._prepare_tree()
+        state = dirstate.DirState.from_tree(tree, 'dirstate',
+            UppercaseSHA1Provider())
+        self.addCleanup(state.unlock)
+        expected_sha = osutils.sha_string(text.upper() + "foo")
+        entry = state._get_entry(0, path_utf8='a file')
+        state._sha_cutoff_time()
+        state._cutoff_time += 10
+        sha1 = self.update_entry(state, entry, 'tree/a file',
+                                 os.lstat('tree/a file'))
+        self.assertEqual(expected_sha, sha1)
 
-    _test_needs_features = [CompiledDirstateHelpersFeature]
+    def test_sha1provider_stat_and_sha1_used(self):
+        tree, text = self._prepare_tree()
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        state = tree._current_dirstate()
+        state._sha1_provider = UppercaseSHA1Provider()
+        # If we used the standard provider, it would look like nothing has
+        # changed
+        file_ids_changed = [change[0] for change
+                            in tree.iter_changes(tree.basis_tree())]
+        self.assertEqual(['a-file-id'], file_ids_changed)
+
+
+class UppercaseSHA1Provider(dirstate.SHA1Provider):
+    """A custom SHA1Provider."""
+
+    def sha1(self, abspath):
+        return self.stat_and_sha1(abspath)[1]
+
+    def stat_and_sha1(self, abspath):
+        file_obj = file(abspath, 'rb')
+        try:
+            statvalue = os.fstat(file_obj.fileno())
+            text = ''.join(file_obj.readlines())
+            sha1 = osutils.sha_string(text.upper() + "foo")
+        finally:
+            file_obj.close()
+        return statvalue, sha1
+
+
+class TestProcessEntry(test_dirstate.TestCaseWithDirState):
+
+    # Set by load_tests
+    _process_entry = None
+
+    def setUp(self):
+        super(TestProcessEntry, self).setUp()
+        orig = dirstate._process_entry
+        def cleanup():
+            dirstate._process_entry = orig
+        self.addCleanup(cleanup)
+        dirstate._process_entry = self._process_entry
+
+    def assertChangedFileIds(self, expected, tree):
+        tree.lock_read()
+        try:
+            file_ids = [info[0] for info
+                        in tree.iter_changes(tree.basis_tree())]
+        finally:
+            tree.unlock()
+        self.assertEqual(sorted(expected), sorted(file_ids))
+
+    def test_simple_changes(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        tree.add(['file'], ['file-id'])
+        self.assertChangedFileIds([tree.get_root_id(), 'file-id'], tree)
+        tree.commit('one')
+        self.assertChangedFileIds([], tree)
+
+    def test_sha1provider_stat_and_sha1_used(self):
+        tree = self.make_branch_and_tree('tree')
+        self.build_tree(['tree/file'])
+        tree.add(['file'], ['file-id'])
+        tree.commit('one')
+        tree.lock_write()
+        self.addCleanup(tree.unlock)
+        state = tree._current_dirstate()
+        state._sha1_provider = UppercaseSHA1Provider()
+        self.assertChangedFileIds(['file-id'], tree)
 
-    def set_update_entry(self):
-        from bzrlib._dirstate_helpers_c import update_entry
-        self.update_entry = update_entry
