@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import sys
 from urllib import quote
@@ -154,34 +154,38 @@ class TestInfo(tests.TestCaseWithTransport):
                     ' rich-root or rich-root-pack'
             elif key in ('knit', 'metaweave'):
                 expected = 'knit or metaweave'
-            elif key in ('development-wt5', 'development-wt5-rich-root'):
-                expected = 'development-wt5 or development-wt5-rich-root'
+            elif key in ('1.14', '1.14-rich-root'):
+                expected = '1.14 or 1.14-rich-root'
             self.assertCheckoutDescription(key, expected)
 
     def test_describe_branch_format(self):
         for key in bzrdir.format_registry.keys():
             if key in bzrdir.format_registry.aliases():
                 continue
+            if bzrdir.format_registry.get_info(key).hidden:
+                continue
             expected = None
             if key in ('dirstate', 'knit'):
                 expected = 'dirstate or knit'
-            elif key in ('1.9', 'development-wt5'):
-                expected = '1.9 or development-wt5'
-            elif key in ('1.9-rich-root', 'development-wt5-rich-root'):
-                expected = '1.9-rich-root or development-wt5-rich-root'
+            elif key in ('1.9', '1.14'):
+                expected = '1.14 or 1.9'
+            elif key in ('1.9-rich-root', '1.14-rich-root'):
+                expected = '1.14-rich-root or 1.9-rich-root'
             self.assertBranchDescription(key, expected)
 
     def test_describe_repo_format(self):
         for key in bzrdir.format_registry.keys():
             if key in bzrdir.format_registry.aliases():
                 continue
+            if bzrdir.format_registry.get_info(key).hidden:
+                continue
             expected = None
             if key in ('dirstate', 'knit', 'dirstate-tags'):
                 expected = 'dirstate or dirstate-tags or knit'
-            elif key in ('1.9', 'development-wt5'):
-                expected = '1.9 or development-wt5'
-            elif key in ('1.9-rich-root', 'development-wt5-rich-root'):
-                expected = '1.9-rich-root or development-wt5-rich-root'
+            elif key in ('1.9', '1.14'):
+                expected = '1.14 or 1.9'
+            elif key in ('1.9-rich-root', '1.14-rich-root'):
+                expected = '1.14-rich-root or 1.9-rich-root'
             self.assertRepoDescription(key, expected)
 
         format = bzrdir.format_registry.make_bzrdir('metaweave')
