@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for plugins"""
 
@@ -31,7 +31,6 @@ import bzrlib.plugin
 import bzrlib.plugins
 import bzrlib.commands
 import bzrlib.help
-from bzrlib.symbol_versioning import one_three
 from bzrlib.tests import (
     TestCase,
     TestCaseInTempDir,
@@ -455,40 +454,6 @@ class TestPluginHelp(TestCaseInTempDir):
                 delattr(bzrlib.plugins, 'myplug')
 
 
-class TestPluginFromZip(TestCaseInTempDir):
-
-    def make_zipped_plugin(self, zip_name, filename):
-        z = zipfile.ZipFile(zip_name, 'w')
-        z.writestr(filename, PLUGIN_TEXT)
-        z.close()
-
-    def check_plugin_load(self, zip_name, plugin_name):
-        self.assertFalse(plugin_name in dir(bzrlib.plugins),
-                         'Plugin already loaded')
-        old_path = bzrlib.plugins.__path__
-        try:
-            # this is normally done by load_plugins -> set_plugins_path
-            bzrlib.plugins.__path__ = [zip_name]
-            self.applyDeprecated(one_three,
-                bzrlib.plugin.load_from_zip, zip_name)
-            self.assertTrue(plugin_name in dir(bzrlib.plugins),
-                            'Plugin is not loaded')
-        finally:
-            # unregister plugin
-            if getattr(bzrlib.plugins, plugin_name, None):
-                delattr(bzrlib.plugins, plugin_name)
-                del sys.modules['bzrlib.plugins.' + plugin_name]
-            bzrlib.plugins.__path__ = old_path
-
-    def test_load_module(self):
-        self.make_zipped_plugin('./test.zip', 'ziplug.py')
-        self.check_plugin_load('./test.zip', 'ziplug')
-
-    def test_load_package(self):
-        self.make_zipped_plugin('./test.zip', 'ziplug/__init__.py')
-        self.check_plugin_load('./test.zip', 'ziplug')
-
-
 class TestSetPluginsPath(TestCase):
 
     def test_set_plugins_path(self):
@@ -668,7 +633,7 @@ class TestPluginPaths(tests.TestCase):
         path = plugin.get_standard_plugins_path()
         self.assertEqual(plugin.get_default_plugin_path(), path[0])
         for directory in path:
-            self.assertNotContainsRe(r'\\/$', directory)
+            self.assertNotContainsRe(directory, r'\\/$')
         try:
             from distutils.sysconfig import get_python_lib
         except ImportError:
