@@ -14,7 +14,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """These tests are tests about the source code of bzrlib itself.
 
@@ -42,9 +42,9 @@ from bzrlib.tests import (
 
 # Files which are listed here will be skipped when testing for Copyright (or
 # GPL) statements.
-COPYRIGHT_EXCEPTIONS = ['bzrlib/lsprof.py']
+COPYRIGHT_EXCEPTIONS = ['bzrlib/lsprof.py', 'bzrlib/_bencode_py.py']
 
-LICENSE_EXCEPTIONS = ['bzrlib/lsprof.py']
+LICENSE_EXCEPTIONS = ['bzrlib/lsprof.py', 'bzrlib/_bencode_py.py']
 # Technically, 'bzrlib/lsprof.py' should be 'bzrlib/util/lsprof.py',
 # (we do not check bzrlib/util/, since that is code bundled from elsewhere)
 # but for compatibility with previous releases, we don't want to move it.
@@ -109,13 +109,15 @@ class TestSource(TestSourceHelper):
                               % source_dir)
         return source_dir
 
-    def get_source_files(self):
+    def get_source_files(self, extensions=None):
         """Yield all source files for bzr and bzrlib
 
         :param our_files_only: If true, exclude files from included libraries
             or plugins.
         """
         bzrlib_dir = self.get_bzrlib_dir()
+        if extensions is None:
+            extensions = ('.py',)
 
         # This is the front-end 'bzr' script
         bzr_path = self.get_bzr_path()
@@ -126,12 +128,16 @@ class TestSource(TestSourceHelper):
                 if d.endswith('.tmp'):
                     dirs.remove(d)
             for f in files:
-                if not f.endswith('.py'):
+                for extension in extensions:
+                    if f.endswith(extension):
+                        break
+                else:
+                    # Did not match the accepted extensions
                     continue
                 yield osutils.pathjoin(root, f)
 
-    def get_source_file_contents(self):
-        for fname in self.get_source_files():
+    def get_source_file_contents(self, extensions=None):
+        for fname in self.get_source_files(extensions=extensions):
             f = open(fname, 'rb')
             try:
                 text = f.read()
@@ -175,9 +181,7 @@ class TestSource(TestSourceHelper):
                           % filename)
 
     def test_copyright(self):
-        """Test that all .py files have a valid copyright statement"""
-        # These are files which contain a different copyright statement
-        # and that is okay.
+        """Test that all .py and .pyx files have a valid copyright statement"""
         incorrect = []
 
         copyright_re = re.compile('#\\s*copyright.*(?=\n)', re.I)
@@ -187,7 +191,8 @@ class TestSource(TestSourceHelper):
             r'.*Canonical Ltd' # And containing 'Canonical Ltd'
             )
 
-        for fname, text in self.get_source_file_contents():
+        for fname, text in self.get_source_file_contents(
+                extensions=('.py', '.pyx')):
             if self.is_copyright_exception(fname):
                 continue
             match = copyright_canonical_re.search(text)
@@ -222,7 +227,7 @@ class TestSource(TestSourceHelper):
             self.fail('\n'.join(help_text))
 
     def test_gpl(self):
-        """Test that all .py files have a GPL disclaimer"""
+        """Test that all .py and .pyx files have a GPL disclaimer."""
         incorrect = []
 
         gpl_txt = """
@@ -238,11 +243,12 @@ class TestSource(TestSourceHelper):
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 """
         gpl_re = re.compile(re.escape(gpl_txt), re.MULTILINE)
 
-        for fname, text in self.get_source_file_contents():
+        for fname, text in self.get_source_file_contents(
+                extensions=('.py', '.pyx')):
             if self.is_license_exception(fname):
                 continue
             if not gpl_re.search(text):
@@ -277,20 +283,22 @@ class TestSource(TestSourceHelper):
     def test_coding_style(self):
         """Check if bazaar code conforms to some coding style conventions.
 
-        Currently we check for:
+        Currently we assert that the following is not present:
          * any tab characters
-         * trailing white space
          * non-unix newlines
          * no newline at end of files
+
+        Print how many files have
+         * trailing white space
          * lines longer than 79 chars
-           (only print how many files and lines are in violation)
         """
         tabs = {}
         trailing_ws = {}
         illegal_newlines = {}
         long_lines = {}
         no_newline_at_eof = []
-        for fname, text in self.get_source_file_contents():
+        for fname, text in self.get_source_file_contents(
+                extensions=('.py', '.pyx')):
             if not self.is_our_code(fname):
                 continue
             lines = text.splitlines(True)
@@ -313,9 +321,9 @@ class TestSource(TestSourceHelper):
                 'Tab characters were found in the following source files.'
                 '\nThey should either be replaced by "\\t" or by spaces:'))
         if trailing_ws:
-            problems.append(self._format_message(trailing_ws,
-                'Trailing white space was found in the following source files:'
-                ))
+            print ("There are %i lines with trailing white space in %i files."
+                % (sum([len(lines) for f, lines in trailing_ws.items()]),
+                    len(trailing_ws)))
         if illegal_newlines:
             problems.append(self._format_message(illegal_newlines,
                 'Non-unix newlines were found in the following source files:'))
