@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for Transport implementations.
 
@@ -155,12 +155,17 @@ class TransportTests(TestTransportImplementation):
         self.assertEqual(True, t.has('a'))
         self.assertEqual(False, t.has('c'))
         self.assertEqual(True, t.has(urlutils.escape('%')))
-        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-                [True, True, False, False, True, False, True, False])
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd',
+                                           'e', 'f', 'g', 'h'])),
+                         [True, True, False, False,
+                          True, False, True, False])
         self.assertEqual(True, t.has_any(['a', 'b', 'c']))
-        self.assertEqual(False, t.has_any(['c', 'd', 'f', urlutils.escape('%%')]))
-        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
-                [True, True, False, False, True, False, True, False])
+        self.assertEqual(False, t.has_any(['c', 'd', 'f',
+                                           urlutils.escape('%%')]))
+        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd',
+                                                'e', 'f', 'g', 'h']))),
+                         [True, True, False, False,
+                          True, False, True, False])
         self.assertEqual(False, t.has_any(['c', 'c', 'c']))
         self.assertEqual(True, t.has_any(['b', 'b', 'b']))
 
@@ -669,6 +674,16 @@ class TransportTests(TestTransportImplementation):
             for f in files:
                 self.assertTransportMode(temp_transport, f, mode)
 
+    def test_create_prefix(self):
+        t = self.get_transport()
+        sub = t.clone('foo').clone('bar')
+        try:
+            sub.create_prefix()
+        except TransportNotPossible:
+            self.assertTrue(t.is_readonly())
+        else:
+            self.assertTrue(t.has('foo/bar'))
+
     def test_append_file(self):
         t = self.get_transport()
 
@@ -1495,16 +1510,10 @@ class TransportTests(TestTransportImplementation):
         transport.put_bytes('foo', 'bar')
         transport3 = self.get_transport()
         self.check_transport_contents('bar', transport3, 'foo')
-        # its base should be usable. XXX: This is true only if we don't use
-        # auhentication, otherwise 'base' doesn't mention the password and we
-        # can't access it anymore since the password is lost (it *could* be
-        # mentioned in the url given by the test server) --vila 090226
-        transport4 = get_transport(transport.base)
-        self.check_transport_contents('bar', transport4, 'foo')
 
         # now opening at a relative url should give use a sane result:
         transport.mkdir('newdir')
-        transport5 = get_transport(transport.base + "newdir")
+        transport5 = self.get_transport('newdir')
         transport6 = transport5.clone('..')
         self.check_transport_contents('bar', transport6, 'foo')
 
