@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from cStringIO import StringIO
 
@@ -21,7 +21,7 @@ from bzrlib import (
     inventory,
     xml7,
     xml8,
-    xml_serializer,
+    serializer,
     )
 from bzrlib.tests import TestCase
 from bzrlib.inventory import Inventory, InventoryEntry
@@ -492,23 +492,12 @@ class TestSerializer(TestCase):
 
         self.assertEqual(len(expected), len(actual))
 
-    def test_registry(self):
-        self.assertIs(serializer_v4,
-                      xml_serializer.format_registry.get('4'))
-        self.assertIs(bzrlib.xml5.serializer_v5,
-                      xml_serializer.format_registry.get('5'))
-        self.assertIs(bzrlib.xml6.serializer_v6,
-                      xml_serializer.format_registry.get('6'))
-        self.assertIs(bzrlib.xml7.serializer_v7,
-                      xml_serializer.format_registry.get('7'))
-        self.assertIs(bzrlib.xml8.serializer_v8,
-                      xml_serializer.format_registry.get('8'))
-
 
 class TestEncodeAndEscape(TestCase):
     """Whitebox testing of the _encode_and_escape function."""
 
     def setUp(self):
+        TestCase.setUp(self)
         # Keep the cache clear before and after the test
         bzrlib.xml8._ensure_utf8_re()
         bzrlib.xml8._clear_cache()
