@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Test that all Tree's implement get_file_mtime"""
 
@@ -29,7 +29,7 @@ class TestGetFileMTime(TestCaseWithTree):
         tree.add(['one'], ['one-id'])
         return self._convert_tree(tree)
 
-    def test_get_symlink_target(self):
+    def test_get_file_mtime(self):
         now = time.time()
         tree = self.get_basic_tree()
         tree.lock_read()
