@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tests for the generic Tree.walkdirs interface."""
 
@@ -26,44 +26,31 @@ from bzrlib.tests.tree_implementations import TestCaseWithTree
 class TestWalkdirs(TestCaseWithTree):
 
     def get_all_subdirs_expected(self, tree, symlinks):
+        dirblocks = [
+            (('', tree.path2id('')),
+             [('0file', '0file', 'file', None, '2file', 'file'),
+              ('1top-dir', '1top-dir', 'directory', None,
+               '1top-dir', 'directory'),
+              (u'2utf\u1234file', u'2utf\u1234file', 'file', None,
+               u'0utf\u1234file'.encode('utf8'), 'file'),
+              ]),
+            (('1top-dir', '1top-dir'),
+             [('1top-dir/0file-in-1topdir', '0file-in-1topdir',
+               'file', None, '1file-in-1topdir', 'file'),
+              ('1top-dir/1dir-in-1topdir', '1dir-in-1topdir',
+               'directory', None, '0dir-in-1topdir', 'directory'),
+              ]),
+            (('1top-dir/1dir-in-1topdir', '0dir-in-1topdir'),
+             []),
+            ]
         if symlinks:
-            return [
-                (('', tree.path2id('')),
-                [
-                 ('0file', '0file', 'file', None, '2file', 'file'),
-                 ('1top-dir', '1top-dir', 'directory', None, '1top-dir', 'directory'),
-                 (u'2utf\u1234file', u'2utf\u1234file', 'file', None,
-                                         u'0utf\u1234file'.encode('utf8'), 'file'),
-                 ('symlink', 'symlink', 'symlink', None, 'symlink', 'symlink')
-                ]),
-                (('1top-dir', '1top-dir'),
-                [('1top-dir/0file-in-1topdir', '0file-in-1topdir', 'file', None, '1file-in-1topdir', 'file'),
-                 ('1top-dir/1dir-in-1topdir', '1dir-in-1topdir', 'directory', None, '0dir-in-1topdir', 'directory'),
-                ]),
-                (('1top-dir/1dir-in-1topdir', '0dir-in-1topdir'),
-                [
-                ]),
-                ]
-        else:
-            return [
-                (('', tree.path2id('')),
-                [
-                 ('0file', '0file', 'file', None, '2file', 'file'),
-                 ('1top-dir', '1top-dir', 'directory', None, '1top-dir', 'directory'),
-                 (u'2utf\u1234file', u'2utf\u1234file', 'file', None,
-                                         u'0utf\u1234file'.encode('utf8'), 'file'),
-                ]),
-                (('1top-dir', '1top-dir'),
-                [('1top-dir/0file-in-1topdir', '0file-in-1topdir', 'file', None, '1file-in-1topdir', 'file'),
-                 ('1top-dir/1dir-in-1topdir', '1dir-in-1topdir', 'directory', None, '0dir-in-1topdir', 'directory'),
-                ]),
-                (('1top-dir/1dir-in-1topdir', '0dir-in-1topdir'),
-                [
-                ]),
-                ]
+            dirblocks[0][1].append(('symlink', 'symlink', 'symlink', None,
+                                    'symlink', 'symlink'))
+        return dirblocks
 
     def test_walkdir_root(self):
-        tree = self.get_tree_with_subdirs_and_all_supported_content_types(has_symlinks())
+        tree = self.get_tree_with_subdirs_and_all_supported_content_types(
+            has_symlinks())
         tree.lock_read()
         expected_dirblocks = self.get_all_subdirs_expected(tree, has_symlinks())
         # test that its iterable by iterating
