@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Messages and logging for bazaar-ng.
 
@@ -329,21 +329,6 @@ def pop_log_file((magic, old_handlers, new_handler, old_trace_file, new_trace_fi
     new_trace_file.flush()
 
 
-@symbol_versioning.deprecated_function(symbol_versioning.one_two)
-def enable_test_log(to_file):
-    """Redirect logging to a temporary file for a test
-
-    :returns: an opaque reference that should be passed to disable_test_log
-    after the test completes.
-    """
-    return push_log_file(to_file)
-
-
-@symbol_versioning.deprecated_function(symbol_versioning.one_two)
-def disable_test_log(memento):
-    return pop_log_file(memento)
-
-
 def log_exception_quietly():
     """Log the last exception to the trace file only.
 
@@ -398,15 +383,6 @@ def is_verbose():
     return _verbosity_level > 0
 
 
-@symbol_versioning.deprecated_function(symbol_versioning.one_two)
-def disable_default_logging():
-    """Turn off default log handlers.
-
-    Don't call this method, use _push_log_file and _pop_log_file instead.
-    """
-    pass
-
-
 def debug_memory(message='', short=True):
     """Write out a memory dump."""
     if sys.platform == 'win32':
