@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """Tree classes, representing directory at point in time.
 """
@@ -24,7 +24,9 @@ from cStringIO import StringIO
 import bzrlib
 from bzrlib import (
     conflicts as _mod_conflicts,
+    debug,
     delta,
+    filters,
     osutils,
     revision as _mod_revision,
     rules,
@@ -95,10 +97,6 @@ class Tree(object):
             want_unversioned=want_unversioned,
             )
 
-    @symbol_versioning.deprecated_method(symbol_versioning.one_three)
-    def _iter_changes(self, *args, **kwargs):
-        return self.iter_changes(*args, **kwargs)
-
     def iter_changes(self, from_tree, include_unchanged=False,
                      specific_files=None, pb=None, extra_trees=None,
                      require_versioned=True, want_unversioned=False):
@@ -204,9 +202,10 @@ class Tree(object):
             specific_file_ids=specific_file_ids)
 
     def iter_references(self):
-        for path, entry in self.iter_entries_by_dir():
-            if entry.kind == 'tree-reference':
-                yield path, entry.file_id
+        if self.supports_tree_reference():
+            for path, entry in self.iter_entries_by_dir():
+                if entry.kind == 'tree-reference':
+                    yield path, entry.file_id
 
     def kind(self, file_id):
         raise NotImplementedError("Tree subclass %s must implement kind"
@@ -264,6 +263,20 @@ class Tree(object):
         """
         raise NotImplementedError(self.get_file)
 
+    def get_file_with_stat(self, file_id, path=None):
+        """Get a file handle and stat object for file_id.
+
+        The default implementation returns (self.get_file, None) for backwards
+        compatibility.
+
+        :param file_id: The file id to read.
+        :param path: The path of the file, if it is known.
+        :return: A tuple (file_handle, stat_value_or_None). If the tree has
+            no stat facility, or need for a stat cache feedback during commit,
+            it may return None for the second element of the tuple.
+        """
+        return (self.get_file(file_id, path), None)
+
     def get_file_text(self, file_id, path=None):
         """Return the byte content of a file.
 
@@ -546,12 +559,6 @@ class Tree(object):
         for child in getattr(entry, 'children', {}).itervalues():
             yield child.file_id
 
-    @symbol_versioning.deprecated_method(symbol_versioning.one_six)
-    def print_file(self, file_id):
-        """Print file with id `file_id` to stdout."""
-        import sys
-        sys.stdout.write(self.get_file_text(file_id))
-
     def lock_read(self):
         pass
 
@@ -627,8 +634,45 @@ class Tree(object):
     def supports_content_filtering(self):
         return False
 
+    def _content_filter_stack(self, path=None, file_id=None):
+        """The stack of content filters for a path if filtering is supported.
+
+        Readers will be applied in first-to-last order.
+        Writers will be applied in last-to-first order.
+        Either the path or the file-id needs to be provided.
+
+        :param path: path relative to the root of the tree
+            or None if unknown
+        :param file_id: file_id or None if unknown
+        :return: the list of filters - [] if there are none
+        """
+        filter_pref_names = filters._get_registered_names()
+        if len(filter_pref_names) == 0:
+            return []
+        if path is None:
+            path = self.id2path(file_id)
+        prefs = self.iter_search_rules([path], filter_pref_names).next()
+        stk = filters._get_filter_stack_for(prefs)
+        if 'filters' in debug.debug_flags:
+            note("*** %s content-filter: %s => %r" % (path,prefs,stk))
+        return stk
+
+    def _content_filter_stack_provider(self):
+        """A function that returns a stack of ContentFilters.
+
+        The function takes a path (relative to the top of the tree) and a
+        file-id as parameters.
+
+        :return: None if content filtering is not supported by this tree.
+        """
+        if self.supports_content_filtering():
+            return lambda path, file_id: \
+                    self._content_filter_stack(path, file_id)
+        else:
+            return None
+
     def iter_search_rules(self, path_names, pref_names=None,
-        _default_searcher=rules._per_user_searcher):
+        _default_searcher=None):
         """Find the preferences for filenames in a tree.
 
         :param path_names: an iterable of paths to find attributes for.
@@ -638,6 +682,8 @@ class Tree(object):
         :return: an iterator of tuple sequences, one per path-name.
           See _RulesSearcher.get_items for details on the tuple sequence.
         """
+        if _default_searcher is None:
+            _default_searcher = rules._per_user_searcher
         searcher = self._get_rules_searcher(_default_searcher)
         if searcher is not None:
             if pref_names is not None:
@@ -876,7 +922,6 @@ class InterTree(InterObject):
             output. An unversioned file is defined as one with (False, False)
             for the versioned pair.
         """
-        result = []
         lookup_trees = [self.source]
         if extra_trees:
              lookup_trees.extend(extra_trees)
@@ -941,17 +986,17 @@ class InterTree(InterObject):
             if kind[0] != kind[1]:
                 changed_content = True
             elif from_kind == 'file':
-                from_size = self.source._file_size(from_entry, from_stat)
-                to_size = self.target._file_size(to_entry, to_stat)
-                if from_size != to_size:
-                    changed_content = True
-                elif (self.source.get_file_sha1(file_id, from_path, from_stat) !=
+                if (self.source.get_file_sha1(file_id, from_path, from_stat) !=
                     self.target.get_file_sha1(file_id, to_path, to_stat)):
                     changed_content = True
             elif from_kind == 'symlink':
                 if (self.source.get_symlink_target(file_id) !=
                     self.target.get_symlink_target(file_id)):
                     changed_content = True
+                # XXX: Yes, the indentation below is wrong. But fixing it broke
+                # test_merge.TestMergerEntriesLCAOnDisk.
+                # test_nested_tree_subtree_renamed_and_modified. We'll wait for
+                # the fix from bzr.dev -- vila 2009026
                 elif from_kind == 'tree-reference':
                     if (self.source.get_reference_revision(file_id, from_path)
                         != self.target.get_reference_revision(file_id, to_path)):
