@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 from bzrlib import (
     cache_utf8,
@@ -83,7 +83,7 @@ class Serializer_v5(xml6.Serializer_v6):
         In this version, no checking is done.
 
         :param inv: An inventory about to be serialised, to be checked.
-        :raises: AssertionError if an error has occured.
+        :raises: AssertionError if an error has occurred.
         """
 
     def _append_inventory_root(self, append, inv):
