@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 """XML externalization support."""
 
@@ -22,7 +22,7 @@
 # importing this module is fairly slow because it has to load several
 # ElementTree bits
 
-from bzrlib import registry
+from bzrlib.serializer import Serializer
 from bzrlib.trace import mutter, warning
 
 try:
@@ -49,15 +49,8 @@ except ImportError:
 from bzrlib import errors
 
 
-class Serializer(object):
-    """Abstract object serialize/deserialize"""
-
-    def write_inventory(self, inv, f):
-        """Write inventory to a file"""
-        raise NotImplementedError(self.write_inventory)
-
-    def write_inventory_to_string(self, inv):
-        raise NotImplementedError(self.write_inventory_to_string)
+class XMLSerializer(Serializer):
+    """Abstract XML object serialize/deserialize"""
 
     def read_inventory_from_string(self, xml_string, revision_id=None,
                                    entry_cache=None):
@@ -175,13 +168,16 @@ def _escape_cdata(text, encoding=None, replace=None):
 elementtree.ElementTree._escape_cdata = _escape_cdata
 
 
-class SerializerRegistry(registry.Registry):
-    """Registry for serializer objects"""
+def escape_invalid_chars(message):
+    """Escape the XML-invalid characters in a commit message.
 
-
-format_registry = SerializerRegistry()
-format_registry.register_lazy('4', 'bzrlib.xml4', 'serializer_v4')
-format_registry.register_lazy('5', 'bzrlib.xml5', 'serializer_v5')
-format_registry.register_lazy('6', 'bzrlib.xml6', 'serializer_v6')
-format_registry.register_lazy('7', 'bzrlib.xml7', 'serializer_v7')
-format_registry.register_lazy('8', 'bzrlib.xml8', 'serializer_v8')
+    :param message: Commit message to escape
+    :return: tuple with escaped message and number of characters escaped
+    """
+    # Python strings can include characters that can't be
+    # represented in well-formed XML; escape characters that
+    # aren't listed in the XML specification
+    # (http://www.w3.org/TR/REC-xml/#NT-Char).
+    return re.subn(u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
+            lambda match: match.group(0).encode('unicode_escape'),
+            message)
